import pytest
from requests.exceptions import HTTPError

from commercetools import types


def test_category_get_by_id(client):
    category = client.categories.create(
        types.CategoryDraft(
            key="test-category",
            name=types.LocalizedString(en="category"),
            slug=types.LocalizedString(en="something"),
        )
    )

    assert category.id
    assert category.key == "test-category"

    category = client.categories.get_by_id(category.id)
    assert category.id
    assert category.key == "test-category"

    with pytest.raises(HTTPError):
        client.categories.get_by_id("invalid")


def test_category_get_by_key(client):
    category = client.categories.create(
        types.CategoryDraft(
            key="test-category",
            name=types.LocalizedString(en="category"),
            slug=types.LocalizedString(en="something"),
        )
    )

    assert category.id
    assert category.key == "test-category"

    category = client.categories.get_by_key("test-category")
    assert category.id
    assert category.key == "test-category"

    with pytest.raises(HTTPError):
        client.categories.get_by_key("invalid")


def test_category_query(client):
    category = client.categories.create(
        types.CategoryDraft(
            key="test-category1",
            name=types.LocalizedString(en="category"),
            slug=types.LocalizedString(en="something"),
        )
    )
    category = client.categories.create(
        types.CategoryDraft(
            key="test-category2",
            name=types.LocalizedString(en="category"),
            slug=types.LocalizedString(en="something"),
        )
    )

    # single sort query
    result = client.categories.query(sort="id asc", limit=10)
    assert len(result.results) == 2
    assert result.total == 2

    # multiple sort queries
    result = client.categories.query(sort=["id asc", "name asc"])
    assert len(result.results) == 2
    assert result.total == 2


def test_category_update(client):
    """Test the return value of the update methods.

    It doesn't test the actual update itself.
    TODO: See if this is worth testing since we're using a mocking backend
    """
    category = client.categories.create(
        types.CategoryDraft(
            key="test-category",
            slug=types.LocalizedString(nl="nl-slug"),
            name=types.LocalizedString(nl="category"),
        )
    )
    assert category.key == "test-category"

    category = client.categories.update_by_id(
        id=category.id,
        version=category.version,
        actions=[
            types.CategoryChangeSlugAction(slug=types.LocalizedString(nl="nl-slug2"))
        ],
    )
    assert category.key == "test-category"

    category = client.categories.update_by_key(
        key="test-category",
        version=category.version,
        actions=[
            types.CategoryChangeSlugAction(slug=types.LocalizedString(nl="nl-slug2"))
        ],
    )
    assert category.key == "test-category"



import pytest
from requests.exceptions import HTTPError

from commercetools import types


def test_cart_discount_get_by_id(client):
    cart_discount = client.cart_discounts.create(
        types.CartDiscountDraft(
            name=types.LocalizedString({"en": "test discount"}),
            value=types.CartDiscountValueRelative(permyriad=10),
            cart_predicate="",
            sort_order="",
            requires_discount_code=False,
        )
    )

    assert cart_discount.id

    cart_discount = client.cart_discounts.get_by_id(cart_discount.id)
    assert cart_discount.id

    with pytest.raises(HTTPError):
        client.cart_discounts.get_by_id("invalid")


def test_cart_discount_query(client):
    client.cart_discounts.create(
        types.CartDiscountDraft(
            name=types.LocalizedString({"en:": "test discount"}),
            value=types.CartDiscountValueRelative(permyriad=10),
            cart_predicate="",
            sort_order="",
            requires_discount_code=False,
        )
    )
    client.cart_discounts.create(
        types.CartDiscountDraft(
            name=types.LocalizedString({"en:": "test discount"}),
            value=types.CartDiscountValueRelative(permyriad=10),
            cart_predicate="",
            sort_order="",
            requires_discount_code=False,
        )
    )

    # single sort query
    result = client.cart_discounts.query(sort="id asc")
    assert len(result.results) == 2
    assert result.total == 2

    # multiple sort queries
    result = client.cart_discounts.query(sort=["id asc", "name asc"])
    assert len(result.results) == 2
    assert result.total == 2


def test_cart_discount_update(client):
    cart_discount = client.cart_discounts.create(
        types.CartDiscountDraft(
            name=types.LocalizedString(en="en-cart_discount"),
            value=types.CartDiscountValueRelative(permyriad=10),
            is_active=True,
            cart_predicate="",
            sort_order="",
            requires_discount_code=False,
        )
    )
    assert cart_discount.is_active is True

    cart_discount = client.cart_discounts.update_by_id(
        id=cart_discount.id,
        version=cart_discount.version,
        actions=[types.CartDiscountChangeIsActiveAction(is_active=False)],
    )

    assert cart_discount.is_active is False



# DO NOT EDIT! This file is automatically generated
import marshmallow

from commercetools import helpers, types
from commercetools._schemas._common import (
    BaseResourceSchema,
    ReferenceSchema,
    ResourceIdentifierSchema,
)
from commercetools._schemas._type import FieldContainerField

__all__ = [
    "ReviewDraftSchema",
    "ReviewPagedQueryResponseSchema",
    "ReviewRatingStatisticsSchema",
    "ReviewReferenceSchema",
    "ReviewResourceIdentifierSchema",
    "ReviewSchema",
    "ReviewSetAuthorNameActionSchema",
    "ReviewSetCustomFieldActionSchema",
    "ReviewSetCustomTypeActionSchema",
    "ReviewSetCustomerActionSchema",
    "ReviewSetKeyActionSchema",
    "ReviewSetLocaleActionSchema",
    "ReviewSetRatingActionSchema",
    "ReviewSetTargetActionSchema",
    "ReviewSetTextActionSchema",
    "ReviewSetTitleActionSchema",
    "ReviewTransitionStateActionSchema",
    "ReviewUpdateActionSchema",
    "ReviewUpdateSchema",
]


class ReviewDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ReviewDraft`."""

    key = marshmallow.fields.String(allow_none=True, missing=None)
    uniqueness_value = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="uniquenessValue"
    )
    locale = marshmallow.fields.String(allow_none=True, missing=None)
    author_name = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="authorName"
    )
    title = marshmallow.fields.String(allow_none=True, missing=None)
    text = marshmallow.fields.String(allow_none=True, missing=None)
    target = helpers.LazyNestedField(
        nested="commercetools._schemas._product.ProductResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    state = helpers.LazyNestedField(
        nested="commercetools._schemas._state.StateResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    rating = marshmallow.fields.Integer(allow_none=True, missing=None)
    customer = helpers.LazyNestedField(
        nested="commercetools._schemas._customer.CustomerResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ReviewDraft(**data)


class ReviewPagedQueryResponseSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ReviewPagedQueryResponse`."""

    limit = marshmallow.fields.Integer(allow_none=True)
    count = marshmallow.fields.Integer(allow_none=True)
    total = marshmallow.fields.Integer(allow_none=True, missing=None)
    offset = marshmallow.fields.Integer(allow_none=True)
    results = helpers.LazyNestedField(
        nested="commercetools._schemas._review.ReviewSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ReviewPagedQueryResponse(**data)


class ReviewRatingStatisticsSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ReviewRatingStatistics`."""

    average_rating = marshmallow.fields.Integer(
        allow_none=True, data_key="averageRating"
    )
    highest_rating = marshmallow.fields.Integer(
        allow_none=True, data_key="highestRating"
    )
    lowest_rating = marshmallow.fields.Integer(allow_none=True, data_key="lowestRating")
    count = marshmallow.fields.Integer(allow_none=True)
    ratings_distribution = marshmallow.fields.Dict(
        allow_none=True, data_key="ratingsDistribution"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ReviewRatingStatistics(**data)


class ReviewReferenceSchema(ReferenceSchema):
    """Marshmallow schema for :class:`commercetools.types.ReviewReference`."""

    obj = helpers.LazyNestedField(
        nested="commercetools._schemas._review.ReviewSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.ReviewReference(**data)


class ReviewResourceIdentifierSchema(ResourceIdentifierSchema):
    """Marshmallow schema for :class:`commercetools.types.ReviewResourceIdentifier`."""

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.ReviewResourceIdentifier(**data)


class ReviewSchema(BaseResourceSchema):
    """Marshmallow schema for :class:`commercetools.types.Review`."""

    id = marshmallow.fields.String(allow_none=True)
    version = marshmallow.fields.Integer(allow_none=True)
    created_at = marshmallow.fields.DateTime(allow_none=True, data_key="createdAt")
    last_modified_at = marshmallow.fields.DateTime(
        allow_none=True, data_key="lastModifiedAt"
    )
    last_modified_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.LastModifiedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="lastModifiedBy",
    )
    created_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.CreatedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="createdBy",
    )
    key = marshmallow.fields.String(allow_none=True, missing=None)
    uniqueness_value = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="uniquenessValue"
    )
    locale = marshmallow.fields.String(allow_none=True, missing=None)
    author_name = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="authorName"
    )
    title = marshmallow.fields.String(allow_none=True, missing=None)
    text = marshmallow.fields.String(allow_none=True, missing=None)
    target = helpers.LazyNestedField(
        nested="commercetools._schemas._product.ProductReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    included_in_statistics = marshmallow.fields.Bool(
        allow_none=True, data_key="includedInStatistics"
    )
    rating = marshmallow.fields.Integer(allow_none=True, missing=None)
    state = helpers.LazyNestedField(
        nested="commercetools._schemas._state.StateReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    customer = helpers.LazyNestedField(
        nested="commercetools._schemas._customer.CustomerReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.Review(**data)


class ReviewUpdateActionSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ReviewUpdateAction`."""

    action = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ReviewUpdateAction(**data)


class ReviewUpdateSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ReviewUpdate`."""

    version = marshmallow.fields.Integer(allow_none=True)
    actions = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("action", "action"),
            discriminator_schemas={
                "setAuthorName": "commercetools._schemas._review.ReviewSetAuthorNameActionSchema",
                "setCustomField": "commercetools._schemas._review.ReviewSetCustomFieldActionSchema",
                "setCustomType": "commercetools._schemas._review.ReviewSetCustomTypeActionSchema",
                "setCustomer": "commercetools._schemas._review.ReviewSetCustomerActionSchema",
                "setKey": "commercetools._schemas._review.ReviewSetKeyActionSchema",
                "setLocale": "commercetools._schemas._review.ReviewSetLocaleActionSchema",
                "setRating": "commercetools._schemas._review.ReviewSetRatingActionSchema",
                "setTarget": "commercetools._schemas._review.ReviewSetTargetActionSchema",
                "setText": "commercetools._schemas._review.ReviewSetTextActionSchema",
                "setTitle": "commercetools._schemas._review.ReviewSetTitleActionSchema",
                "transitionState": "commercetools._schemas._review.ReviewTransitionStateActionSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        ),
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ReviewUpdate(**data)


class ReviewSetAuthorNameActionSchema(ReviewUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ReviewSetAuthorNameAction`."""

    author_name = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="authorName"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ReviewSetAuthorNameAction(**data)


class ReviewSetCustomFieldActionSchema(ReviewUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ReviewSetCustomFieldAction`."""

    name = marshmallow.fields.String(allow_none=True)
    value = marshmallow.fields.Raw(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ReviewSetCustomFieldAction(**data)


class ReviewSetCustomTypeActionSchema(ReviewUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ReviewSetCustomTypeAction`."""

    type = helpers.LazyNestedField(
        nested="commercetools._schemas._type.TypeResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    fields = FieldContainerField(allow_none=True, missing=None)  # type: ignore

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ReviewSetCustomTypeAction(**data)


class ReviewSetCustomerActionSchema(ReviewUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ReviewSetCustomerAction`."""

    customer = helpers.LazyNestedField(
        nested="commercetools._schemas._customer.CustomerResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ReviewSetCustomerAction(**data)


class ReviewSetKeyActionSchema(ReviewUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ReviewSetKeyAction`."""

    key = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ReviewSetKeyAction(**data)


class ReviewSetLocaleActionSchema(ReviewUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ReviewSetLocaleAction`."""

    locale = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ReviewSetLocaleAction(**data)


class ReviewSetRatingActionSchema(ReviewUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ReviewSetRatingAction`."""

    rating = marshmallow.fields.Integer(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ReviewSetRatingAction(**data)


class ReviewSetTargetActionSchema(ReviewUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ReviewSetTargetAction`."""

    target = helpers.LazyNestedField(
        nested="commercetools._schemas._product.ProductResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ReviewSetTargetAction(**data)


class ReviewSetTextActionSchema(ReviewUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ReviewSetTextAction`."""

    text = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ReviewSetTextAction(**data)


class ReviewSetTitleActionSchema(ReviewUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ReviewSetTitleAction`."""

    title = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ReviewSetTitleAction(**data)


class ReviewTransitionStateActionSchema(ReviewUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ReviewTransitionStateAction`."""

    state = helpers.LazyNestedField(
        nested="commercetools._schemas._state.StateResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    force = marshmallow.fields.Bool(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ReviewTransitionStateAction(**data)



from commercetools import types
from commercetools._schemas._review import ReviewDraftSchema


def test_create_review(client):
    review = client.reviews.create(draft=types.ReviewDraft(rating=3))

    assert review.id
    assert review.rating == 3


def test_get_by_id(client):
    review = client.reviews.create(draft=ReviewDraftSchema().dump({}))
    assert review.id

    review = client.reviews.get_by_id(review.id)
    assert review


def test_get_by_key(client):
    review = client.reviews.create(
        draft=ReviewDraftSchema().dump({"key": "test-review"})
    )
    assert review.key

    review = client.reviews.get_by_key(review.key)
    assert review


def test_query(client):
    client.reviews.create(draft=types.ReviewDraft(key="test-review-1"))
    client.reviews.create(draft=types.ReviewDraft(key="test-review-2"))

    result = client.reviews.query(sort="id asc", limit=10)
    assert len(result.results) == 2
    assert result.total == 2

    result = client.reviews.query(sort=["id asc", "name asc"], limit=1)
    assert len(result.results) == 1
    assert result.total == 2


def test_delete_by_id(client):
    review = client.reviews.create(draft=types.ReviewDraft())
    assert review.id
    assert client.reviews.delete_by_id(review.id, version=review.version)


def test_delete_by_key(client):
    review = client.reviews.create(
        draft=ReviewDraftSchema().dump({"key": "test-review"})
    )
    assert review.key
    assert client.reviews.delete_by_key(review.key, version=review.version)



import uuid

import pytest
import requests_mock
from requests.exceptions import HTTPError

from commercetools import types


def test_products_create(client):
    custom_type = client.types.create(
        types.TypeDraft(
            name=types.LocalizedString(en="myType"),
            key="my-type",
            resource_type_ids=[types.ResourceTypeId.ASSET],
            field_definitions=[
                types.FieldDefinition(
                    name="foo",
                    type=types.CustomFieldStringType(),
                    label=types.LocalizedString(en="foo"),
                    required=False,
                )
            ],
        )
    )
    assert custom_type.id

    draft = types.ProductDraft(
        key="test-product",
        publish=True,
        name=types.LocalizedString(en=f"my-product"),
        slug=types.LocalizedString(en=f"my-product"),
        product_type=types.ProductTypeResourceIdentifier(key="dummy"),
        master_variant=types.ProductVariantDraft(
            assets=[
                types.AssetDraft(
                    sources=[],
                    name=types.LocalizedString(en="something"),
                    custom=types.CustomFieldsDraft(
                        type=types.TypeResourceIdentifier(id=custom_type.id),
                        fields=types.FieldContainer(foo="bar"),
                    ),
                )
            ],
            prices=[
                types.PriceDraft(
                    value=types.CentPrecisionMoneyDraft(
                        cent_amount=1000, currency_code="EUR"
                    ),
                    country="NL",
                )
            ],
        ),
    )
    product = client.products.create(draft)
    assert product.id
    assert product.master_data.current.master_variant.assets
    assert product.master_data.current.master_variant.prices


def test_products_get_by_id(client):
    product = client.products.create(
        types.ProductDraft(
            key="test-product",
            product_type=types.ProductTypeResourceIdentifier(key="dummy"),
            name=types.LocalizedString(en="my-product"),
            slug=types.LocalizedString(en="my-product"),
            publish=True,
        )
    )

    assert product.id
    assert product.key == "test-product"

    product = client.products.get_by_id(product.id)
    assert product.id
    assert product.key == "test-product"

    with pytest.raises(HTTPError) as e:
        client.products.get_by_id("invalid")


def test_products_get_by_key(client):
    product = client.products.create(
        types.ProductDraft(
            key="test-product",
            product_type=types.ProductTypeResourceIdentifier(key="dummy"),
            name=types.LocalizedString(en="my-product"),
            slug=types.LocalizedString(en="my-product"),
            publish=True,
        )
    )

    assert product.id
    assert product.key == "test-product"

    product = client.products.get_by_key("test-product")
    assert product.id
    assert product.key == "test-product"

    with pytest.raises(HTTPError) as e:
        client.products.get_by_key("invalid")


def test_product_query(client):
    client.products.create(
        types.ProductDraft(
            key=f"product-1",
            product_type=types.ProductTypeResourceIdentifier(key="dummy"),
            name=types.LocalizedString(en=f"my-product-1"),
            slug=types.LocalizedString(en=f"my-product-1"),
            publish=True,
        )
    )
    client.products.create(
        types.ProductDraft(
            key=f"product-2",
            product_type=types.ProductTypeResourceIdentifier(key="dummy"),
            name=types.LocalizedString(en=f"my-product-2"),
            slug=types.LocalizedString(en=f"my-product-2"),
            publish=True,
        )
    )

    # single sort query
    result = client.products.query(sort="id asc", limit=2)
    assert len(result.results) == 2
    assert result.total == 2

    # multiple sort queries
    result = client.products.query(sort=["id asc", "name asc"])
    assert len(result.results) == 2
    assert result.total == 2


def test_product_query_where(client):
    client.products.create(
        types.ProductDraft(
            key="test-product1",
            name=types.LocalizedString(en=f"my-product-1"),
            slug=types.LocalizedString(en=f"my-product-1"),
            product_type=types.ProductTypeResourceIdentifier(key="dummy"),
            master_variant=types.ProductVariantDraft(
                prices=[
                    types.PriceDraft(
                        country="NL",
                        value=types.CentPrecisionMoneyDraft(
                            cent_amount=8750, currency_code="EUR"
                        ),
                    )
                ]
            ),
        )
    )
    client.products.create(
        types.ProductDraft(
            key="test-product-2",
            name=types.LocalizedString(en=f"my-product-1"),
            slug=types.LocalizedString(en=f"my-product-1"),
            product_type=types.ProductTypeResourceIdentifier(key="dummy"),
            master_variant=types.ProductVariantDraft(
                prices=[
                    types.PriceDraft(
                        country="UK",
                        value=types.CentPrecisionMoneyDraft(
                            cent_amount=8750, currency_code="EUR"
                        ),
                    )
                ]
            ),
        )
    )
    client.products.create(
        types.ProductDraft(
            key="test-product2",
            name=types.LocalizedString(en=f"my-product-1"),
            slug=types.LocalizedString(en=f"my-product-1"),
            product_type=types.ProductTypeResourceIdentifier(key="dummy"),
        )
    )

    result = client.products.query(
        where="masterData(staged(masterVariant(prices(country='NL'))))"
    )
    assert len(result.results) == 1
    assert result.total == 1

    result = client.products.query(
        where="masterData(staged(masterVariant(prices(country='UK'))))"
    )
    assert len(result.results) == 1
    assert result.total == 1

    result = client.products.query(
        where="masterData(staged(masterVariant(prices(country='UK' or country='NL'))))"
    )
    assert len(result.results) == 2
    assert result.total == 2


def test_product_update(client):
    """Test the return value of the update methods.

    It doesn't test the actual update itself.
    TODO: See if this is worth testing since we're using a mocking backend
    """
    product = client.products.create(
        types.ProductDraft(
            key="test-product",
            name=types.LocalizedString(en=f"my-product-1"),
            slug=types.LocalizedString(en=f"my-product-1"),
            product_type=types.ProductTypeResourceIdentifier(key="dummy"),
            master_variant=types.ProductVariantDraft(sku="1", key="1"),
        )
    )

    assert uuid.UUID(product.id)
    assert product.key == "test-product"

    product = client.products.update_by_id(
        id=product.id,
        version=product.version,
        actions=[
            types.ProductChangeSlugAction(slug=types.LocalizedString(nl="nl-slug2"))
        ],
    )
    assert product.key == "test-product"
    assert product.master_data.published is False

    product = client.products.update_by_id(
        id=product.id, version=product.version, actions=[types.ProductPublishAction()]
    )
    assert product.master_data.published is True

    assert not product.master_data.current.master_variant.prices
    product = client.products.update_by_id(
        id=product.id,
        version=product.version,
        actions=[
            types.ProductSetPricesAction(
                sku="1",
                prices=[
                    types.PriceDraft(
                        value=types.Money(cent_amount=1000, currency_code="GBP")
                    )
                ],
                staged=False,
            )
        ],
    )

    assert len(product.master_data.current.master_variant.prices) == 1

    product = client.products.update_by_key(
        key="test-product",
        version=product.version,
        actions=[
            types.ProductChangeSlugAction(slug=types.LocalizedString(nl="nl-slug2"))
        ],
    )
    assert product.key == "test-product"


def test_product_update_add_change_price_staged(client):
    product = client.products.create(
        types.ProductDraft(
            key="test-product",
            name=types.LocalizedString(en=f"my-product-1"),
            slug=types.LocalizedString(en=f"my-product-1"),
            product_type=types.ProductTypeResourceIdentifier(key="dummy"),
            master_variant=types.ProductVariantDraft(sku="1", key="1"),
        )
    )

    product = client.products.update_by_id(
        id=product.id,
        version=product.version,
        actions=[
            types.ProductAddPriceAction(
                sku="1",
                price=types.PriceDraft(
                    value=types.Money(cent_amount=1000, currency_code="GBP")
                ),
            )
        ],
    )

    assert product.master_data.current is None
    assert len(product.master_data.staged.master_variant.prices) == 1
    price = product.master_data.staged.master_variant.prices[0]
    assert price.value.cent_amount == 1000
    assert price.value.currency_code == "GBP"

    product = client.products.update_by_id(
        id=product.id,
        version=product.version,
        actions=[
            types.ProductChangePriceAction(
                price_id=price.id,
                price=types.PriceDraft(
                    value=types.Money(cent_amount=3000, currency_code="EUR")
                ),
            )
        ],
    )

    assert product.master_data.current is None
    assert len(product.master_data.staged.master_variant.prices) == 1
    price = product.master_data.staged.master_variant.prices[0]
    assert price.value.cent_amount == 3000
    assert price.value.currency_code == "EUR"


def test_product_update_add_price_current(client):
    product = client.products.create(
        types.ProductDraft(
            key="test-product",
            name=types.LocalizedString(en=f"my-product-1"),
            slug=types.LocalizedString(en=f"my-product-1"),
            product_type=types.ProductTypeResourceIdentifier(key="dummy"),
            master_variant=types.ProductVariantDraft(sku="1", key="1"),
            publish=True,
        )
    )

    product = client.products.update_by_id(
        id=product.id,
        version=product.version,
        actions=[
            types.ProductAddPriceAction(
                sku="1",
                staged=False,
                price=types.PriceDraft(
                    value=types.Money(cent_amount=1000, currency_code="GBP")
                ),
            )
        ],
    )

    assert product.master_data.staged is None
    assert len(product.master_data.current.master_variant.prices) == 1


def test_predicate_var(client):
    with requests_mock.Mocker(real_http=True, case_sensitive=True) as m:

        result = client.products.query(
            where="masterData(staged(masterVariant(prices(country='NL'))))",
            predicate_var={"foo": "bar"},
        )

        assert "var.foo" in m.request_history[0].qs



# DO NOT EDIT! This file is automatically generated
import marshmallow

from commercetools import helpers, types
from commercetools._schemas._common import (
    BaseResourceSchema,
    LocalizedStringField,
    ReferenceSchema,
    ResourceIdentifierSchema,
)
from commercetools._schemas._type import FieldContainerField

__all__ = [
    "CategoryAddAssetActionSchema",
    "CategoryChangeAssetNameActionSchema",
    "CategoryChangeAssetOrderActionSchema",
    "CategoryChangeNameActionSchema",
    "CategoryChangeOrderHintActionSchema",
    "CategoryChangeParentActionSchema",
    "CategoryChangeSlugActionSchema",
    "CategoryDraftSchema",
    "CategoryPagedQueryResponseSchema",
    "CategoryReferenceSchema",
    "CategoryRemoveAssetActionSchema",
    "CategoryResourceIdentifierSchema",
    "CategorySchema",
    "CategorySetAssetCustomFieldActionSchema",
    "CategorySetAssetCustomTypeActionSchema",
    "CategorySetAssetDescriptionActionSchema",
    "CategorySetAssetKeyActionSchema",
    "CategorySetAssetSourcesActionSchema",
    "CategorySetAssetTagsActionSchema",
    "CategorySetCustomFieldActionSchema",
    "CategorySetCustomTypeActionSchema",
    "CategorySetDescriptionActionSchema",
    "CategorySetExternalIdActionSchema",
    "CategorySetKeyActionSchema",
    "CategorySetMetaDescriptionActionSchema",
    "CategorySetMetaKeywordsActionSchema",
    "CategorySetMetaTitleActionSchema",
    "CategoryUpdateActionSchema",
    "CategoryUpdateSchema",
]


class CategoryDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.CategoryDraft`."""

    name = LocalizedStringField(allow_none=True)
    slug = LocalizedStringField(allow_none=True)
    description = LocalizedStringField(allow_none=True, missing=None)
    parent = helpers.LazyNestedField(
        nested="commercetools._schemas._category.CategoryResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    order_hint = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="orderHint"
    )
    external_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="externalId"
    )
    meta_title = LocalizedStringField(
        allow_none=True, missing=None, data_key="metaTitle"
    )
    meta_description = LocalizedStringField(
        allow_none=True, missing=None, data_key="metaDescription"
    )
    meta_keywords = LocalizedStringField(
        allow_none=True, missing=None, data_key="metaKeywords"
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    assets = helpers.LazyNestedField(
        nested="commercetools._schemas._common.AssetDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
    )
    key = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.CategoryDraft(**data)


class CategoryPagedQueryResponseSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.CategoryPagedQueryResponse`."""

    limit = marshmallow.fields.Integer(allow_none=True)
    count = marshmallow.fields.Integer(allow_none=True)
    total = marshmallow.fields.Integer(allow_none=True, missing=None)
    offset = marshmallow.fields.Integer(allow_none=True)
    results = helpers.LazyNestedField(
        nested="commercetools._schemas._category.CategorySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.CategoryPagedQueryResponse(**data)


class CategoryReferenceSchema(ReferenceSchema):
    """Marshmallow schema for :class:`commercetools.types.CategoryReference`."""

    obj = helpers.LazyNestedField(
        nested="commercetools._schemas._category.CategorySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.CategoryReference(**data)


class CategoryResourceIdentifierSchema(ResourceIdentifierSchema):
    """Marshmallow schema for :class:`commercetools.types.CategoryResourceIdentifier`."""

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.CategoryResourceIdentifier(**data)


class CategorySchema(BaseResourceSchema):
    """Marshmallow schema for :class:`commercetools.types.Category`."""

    id = marshmallow.fields.String(allow_none=True)
    version = marshmallow.fields.Integer(allow_none=True)
    created_at = marshmallow.fields.DateTime(allow_none=True, data_key="createdAt")
    last_modified_at = marshmallow.fields.DateTime(
        allow_none=True, data_key="lastModifiedAt"
    )
    last_modified_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.LastModifiedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="lastModifiedBy",
    )
    created_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.CreatedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="createdBy",
    )
    name = LocalizedStringField(allow_none=True)
    slug = LocalizedStringField(allow_none=True)
    description = LocalizedStringField(allow_none=True, missing=None)
    ancestors = helpers.LazyNestedField(
        nested="commercetools._schemas._category.CategoryReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )
    parent = helpers.LazyNestedField(
        nested="commercetools._schemas._category.CategoryReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    order_hint = marshmallow.fields.String(allow_none=True, data_key="orderHint")
    external_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="externalId"
    )
    meta_title = LocalizedStringField(
        allow_none=True, missing=None, data_key="metaTitle"
    )
    meta_description = LocalizedStringField(
        allow_none=True, missing=None, data_key="metaDescription"
    )
    meta_keywords = LocalizedStringField(
        allow_none=True, missing=None, data_key="metaKeywords"
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    assets = helpers.LazyNestedField(
        nested="commercetools._schemas._common.AssetSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
    )
    key = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.Category(**data)


class CategoryUpdateActionSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.CategoryUpdateAction`."""

    action = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CategoryUpdateAction(**data)


class CategoryUpdateSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.CategoryUpdate`."""

    version = marshmallow.fields.Integer(allow_none=True)
    actions = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("action", "action"),
            discriminator_schemas={
                "addAsset": "commercetools._schemas._category.CategoryAddAssetActionSchema",
                "changeAssetName": "commercetools._schemas._category.CategoryChangeAssetNameActionSchema",
                "changeAssetOrder": "commercetools._schemas._category.CategoryChangeAssetOrderActionSchema",
                "changeName": "commercetools._schemas._category.CategoryChangeNameActionSchema",
                "changeOrderHint": "commercetools._schemas._category.CategoryChangeOrderHintActionSchema",
                "changeParent": "commercetools._schemas._category.CategoryChangeParentActionSchema",
                "changeSlug": "commercetools._schemas._category.CategoryChangeSlugActionSchema",
                "removeAsset": "commercetools._schemas._category.CategoryRemoveAssetActionSchema",
                "setAssetCustomField": "commercetools._schemas._category.CategorySetAssetCustomFieldActionSchema",
                "setAssetCustomType": "commercetools._schemas._category.CategorySetAssetCustomTypeActionSchema",
                "setAssetDescription": "commercetools._schemas._category.CategorySetAssetDescriptionActionSchema",
                "setAssetKey": "commercetools._schemas._category.CategorySetAssetKeyActionSchema",
                "setAssetSources": "commercetools._schemas._category.CategorySetAssetSourcesActionSchema",
                "setAssetTags": "commercetools._schemas._category.CategorySetAssetTagsActionSchema",
                "setCustomField": "commercetools._schemas._category.CategorySetCustomFieldActionSchema",
                "setCustomType": "commercetools._schemas._category.CategorySetCustomTypeActionSchema",
                "setDescription": "commercetools._schemas._category.CategorySetDescriptionActionSchema",
                "setExternalId": "commercetools._schemas._category.CategorySetExternalIdActionSchema",
                "setKey": "commercetools._schemas._category.CategorySetKeyActionSchema",
                "setMetaDescription": "commercetools._schemas._category.CategorySetMetaDescriptionActionSchema",
                "setMetaKeywords": "commercetools._schemas._category.CategorySetMetaKeywordsActionSchema",
                "setMetaTitle": "commercetools._schemas._category.CategorySetMetaTitleActionSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        ),
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.CategoryUpdate(**data)


class CategoryAddAssetActionSchema(CategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CategoryAddAssetAction`."""

    asset = helpers.LazyNestedField(
        nested="commercetools._schemas._common.AssetDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    position = marshmallow.fields.Integer(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CategoryAddAssetAction(**data)


class CategoryChangeAssetNameActionSchema(CategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CategoryChangeAssetNameAction`."""

    asset_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="assetId"
    )
    asset_key = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="assetKey"
    )
    name = LocalizedStringField(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CategoryChangeAssetNameAction(**data)


class CategoryChangeAssetOrderActionSchema(CategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CategoryChangeAssetOrderAction`."""

    asset_order = marshmallow.fields.List(
        marshmallow.fields.String(allow_none=True), data_key="assetOrder"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CategoryChangeAssetOrderAction(**data)


class CategoryChangeNameActionSchema(CategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CategoryChangeNameAction`."""

    name = LocalizedStringField(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CategoryChangeNameAction(**data)


class CategoryChangeOrderHintActionSchema(CategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CategoryChangeOrderHintAction`."""

    order_hint = marshmallow.fields.String(allow_none=True, data_key="orderHint")

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CategoryChangeOrderHintAction(**data)


class CategoryChangeParentActionSchema(CategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CategoryChangeParentAction`."""

    parent = helpers.LazyNestedField(
        nested="commercetools._schemas._category.CategoryResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CategoryChangeParentAction(**data)


class CategoryChangeSlugActionSchema(CategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CategoryChangeSlugAction`."""

    slug = LocalizedStringField(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CategoryChangeSlugAction(**data)


class CategoryRemoveAssetActionSchema(CategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CategoryRemoveAssetAction`."""

    asset_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="assetId"
    )
    asset_key = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="assetKey"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CategoryRemoveAssetAction(**data)


class CategorySetAssetCustomFieldActionSchema(CategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CategorySetAssetCustomFieldAction`."""

    asset_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="assetId"
    )
    asset_key = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="assetKey"
    )
    name = marshmallow.fields.String(allow_none=True)
    value = marshmallow.fields.Raw(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CategorySetAssetCustomFieldAction(**data)


class CategorySetAssetCustomTypeActionSchema(CategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CategorySetAssetCustomTypeAction`."""

    asset_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="assetId"
    )
    asset_key = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="assetKey"
    )
    type = helpers.LazyNestedField(
        nested="commercetools._schemas._type.TypeResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    fields = marshmallow.fields.Dict(allow_none=True, missing=None)  # type: ignore

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CategorySetAssetCustomTypeAction(**data)


class CategorySetAssetDescriptionActionSchema(CategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CategorySetAssetDescriptionAction`."""

    asset_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="assetId"
    )
    asset_key = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="assetKey"
    )
    description = LocalizedStringField(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CategorySetAssetDescriptionAction(**data)


class CategorySetAssetKeyActionSchema(CategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CategorySetAssetKeyAction`."""

    asset_id = marshmallow.fields.String(allow_none=True, data_key="assetId")
    asset_key = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="assetKey"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CategorySetAssetKeyAction(**data)


class CategorySetAssetSourcesActionSchema(CategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CategorySetAssetSourcesAction`."""

    asset_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="assetId"
    )
    asset_key = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="assetKey"
    )
    sources = helpers.LazyNestedField(
        nested="commercetools._schemas._common.AssetSourceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CategorySetAssetSourcesAction(**data)


class CategorySetAssetTagsActionSchema(CategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CategorySetAssetTagsAction`."""

    asset_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="assetId"
    )
    asset_key = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="assetKey"
    )
    tags = marshmallow.fields.List(
        marshmallow.fields.String(allow_none=True), missing=None
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CategorySetAssetTagsAction(**data)


class CategorySetCustomFieldActionSchema(CategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CategorySetCustomFieldAction`."""

    name = marshmallow.fields.String(allow_none=True)
    value = marshmallow.fields.Raw(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CategorySetCustomFieldAction(**data)


class CategorySetCustomTypeActionSchema(CategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CategorySetCustomTypeAction`."""

    type = helpers.LazyNestedField(
        nested="commercetools._schemas._type.TypeResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    fields = FieldContainerField(allow_none=True, missing=None)  # type: ignore

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CategorySetCustomTypeAction(**data)


class CategorySetDescriptionActionSchema(CategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CategorySetDescriptionAction`."""

    description = LocalizedStringField(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CategorySetDescriptionAction(**data)


class CategorySetExternalIdActionSchema(CategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CategorySetExternalIdAction`."""

    external_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="externalId"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CategorySetExternalIdAction(**data)


class CategorySetKeyActionSchema(CategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CategorySetKeyAction`."""

    key = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CategorySetKeyAction(**data)


class CategorySetMetaDescriptionActionSchema(CategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CategorySetMetaDescriptionAction`."""

    meta_description = LocalizedStringField(
        allow_none=True, missing=None, data_key="metaDescription"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CategorySetMetaDescriptionAction(**data)


class CategorySetMetaKeywordsActionSchema(CategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CategorySetMetaKeywordsAction`."""

    meta_keywords = LocalizedStringField(
        allow_none=True, missing=None, data_key="metaKeywords"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CategorySetMetaKeywordsAction(**data)


class CategorySetMetaTitleActionSchema(CategoryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CategorySetMetaTitleAction`."""

    meta_title = LocalizedStringField(
        allow_none=True, missing=None, data_key="metaTitle"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CategorySetMetaTitleAction(**data)



from commercetools.predicates import QueryPredicate


def test_simple():
    obj = QueryPredicate(masterVariant__prices__country="NL")
    assert str(obj) == 'masterVariant(prices(country = "NL"))'


def test_two_filters():
    obj = QueryPredicate(
        masterVariant__slug__nl="some-slug", masterVariant__prices__country="NL"
    )
    assert (
        str(obj)
        == 'masterVariant(slug(nl = "some-slug")) AND masterVariant(prices(country = "NL"))'
    )


def test_dict_format():
    obj = QueryPredicate(**{"masterVariant__slug__en-GB": "something"})
    assert str(obj) == 'masterVariant(slug(en-GB = "something"))'


def test_nested_dict():
    obj = QueryPredicate(masterVariant={"slug__en-GB": "something", "prices": 100})
    assert str(obj) == 'masterVariant(slug(en-GB = "something") AND prices = 100)'

    obj = QueryPredicate(supplyChannel={"id": 100, "sku": "S001"})
    assert str(obj) == 'supplyChannel(id = 100 AND sku = "S001")'


def test_is_defined():
    obj = QueryPredicate(
        masterVariant__slug__is_defined=True, masterVariant__prices__is_defined=False
    )
    assert (
        str(obj)
        == "masterVariant(slug is defined) AND masterVariant(prices is not defined)"
    )


def test_gte():
    obj = QueryPredicate(
        masterVariant__slug__nl__exact="some-slug",
        masterVariant__prices__amount__gte=100,
    )
    assert (
        str(obj)
        == 'masterVariant(slug(nl = "some-slug")) AND masterVariant(prices(amount >= 100))'
    )


def test_qp_contains_any():
    obj = QueryPredicate(masterVariant__slug__nl__contains_any=["slug-1", "slug-2"])
    assert str(obj) == 'masterVariant(slug(nl contains any ("slug-1", "slug-2")))'


def test_qp_or():
    obj = QueryPredicate(
        masterVariant__slug=QueryPredicate(nl="test-nl") | QueryPredicate(en="test-en")
    )
    assert str(obj) == 'masterVariant(slug((nl = "test-nl") OR (en = "test-en")))'


def test_qp_and():
    obj = QueryPredicate(
        masterVariant__slug=QueryPredicate(nl="test-nl") & QueryPredicate(en="test-en")
    )
    assert str(obj) == 'masterVariant(slug((nl = "test-nl") AND (en = "test-en")))'


def test_qp_in():
    obj = QueryPredicate(masterVariant__slug__nl__in=["test-nl", "test-en"])
    assert str(obj) == 'masterVariant(slug(nl in ("test-nl", "test-en")))'


def test_qp_nested():
    obj = QueryPredicate(masterVariant__slug__nl="test-nl1") | (
        QueryPredicate(
            masterVariant__slug=(
                QueryPredicate(nl="test-nl2")
                & (QueryPredicate(en="test-en") | QueryPredicate(fr="test-fr"))
            )
        )
    )
    assert str(obj) == (
        '(masterVariant(slug(nl = "test-nl1")))'
        " OR "
        '(masterVariant(slug((nl = "test-nl2") AND ((en = "test-en") OR (fr = "test-fr")))))'
    )


def test_qp_chained():
    obj = (
        QueryPredicate(masterVariant__slug__nl="test-nl1")
        & QueryPredicate(masterVariant__slug__nl="test-nl2")
        & (
            QueryPredicate(masterVariant__prices__is_defined=True)
            | QueryPredicate(masterVariant__prices__amount__gte=100)
        )
    )
    assert str(obj) == (
        '((masterVariant(slug(nl = "test-nl1")))'
        ' AND (masterVariant(slug(nl = "test-nl2"))))'
        " AND ((masterVariant(prices is defined)) OR (masterVariant(prices(amount >= 100))))"
    )



import base64
import typing
import uuid
from urllib.parse import parse_qs

from commercetools.testing.abstract import BaseBackend
from commercetools.testing.utils import create_commercetools_response


class AuthModel:
    def __init__(self):
        self.tokens: typing.List[str] = []

    def add_token(self, token):
        self.tokens.append(token)

    def is_valid(self, client_id, client_secret):
        return True


class AuthBackend(BaseBackend):
    path_prefix = r"/oauth/(?P<path>.*)"
    hostnames = ["auth.sphere.io", "localhost"]
    model_class = AuthModel

    def __init__(self, *args, **kwargs):
        self._expire_time = 172800
        super().__init__()

    def set_expire_time(self, value):
        self._expire_time = value

    @property
    def url_prefix(self):
        return r"/oauth/(?P<path>.*)"

    def urls(self):
        return [("token", "POST", self.token), ("introspect", "POST", self.introspect)]

    def _get_api_client_credentials(
        self, request
    ) -> typing.Tuple[typing.Optional[str], typing.Optional[str]]:
        params = parse_qs(request.body)
        client_id = None
        client_secret = None

        if request.headers.get("Authorization"):
            auth_type, auth_info = request.headers["Authorization"].split()
            if auth_type == "Basic":
                client_id, client_secret = str(base64.b64decode(auth_info)).split(":")
        elif params.get("client_id") and params.get("client_secret"):
            client_id = params.get("client_id")
            client_secret = params.get("client_secret")

        return client_id, client_secret

    def token(self, request):
        client_id, client_secret = self._get_api_client_credentials(request)
        if not client_id or not client_secret:
            response = create_commercetools_response(request, status_code=401)
            return response

        if self.model.is_valid(client_id, client_secret):
            params = parse_qs(request.body)
            scope = params.get("scope", "manage_project:todo")
            token = {
                "access_token": str(uuid.uuid4()),
                "expires_in": self._expire_time,
                "scope": scope,
                "token_type": "Bearer",
            }
            self.model.add_token(token)
            response = create_commercetools_response(request, json=token)
            return response

    def introspect(self, request):
        client_id, client_secret = self._get_api_client_credentials(request)
        if not client_id or not client_secret:
            response = create_commercetools_response(request, status_code=401)
            return response

        if self.model.is_valid(client_id, client_secret):
            token = request.qs.get("token", [None])[0]
            stored_tokens = [
                token_object.get("access_token") for token_object in self.model.tokens
            ]
            if token in stored_tokens:
                status = {
                    "active": True,
                    "scope": "manage_project:todo",
                    "exp": self._expire_time,
                }
            else:
                status = {"active": False}
            response = create_commercetools_response(request, json=status)
            return response



import commercetools
from commercetools import types
from tests.test_service_order import get_test_order


def test_unknown_expand_terms(client: commercetools.Client):
    cart = client.carts.create(types.CartDraft(currency="EUR"))

    order = client.orders.create(
        types.OrderFromCartDraft(id=cart.id, version=1, order_number="test-order"),
        expand="nonExisting",
    )

    assert order.id


def test_optional_expanded_terms(client, commercetools_api):
    order = get_test_order()
    commercetools_api.orders.add_existing(order)

    expanded_order = client.orders.get_by_id(
        order.id, expand="discountCodes[*].discountCode"
    )

    assert expanded_order.id
    assert expanded_order.discount_codes is None


def test_unknown_reference_expand_terms(client, commercetools_api):
    order = get_test_order()
    commercetools_api.orders.add_existing(order)

    expanded_order = client.orders.get_by_id(
        order.id, expand="shippingInfo.shippingMethod"
    )

    assert expanded_order.id
    assert expanded_order.shipping_info.shipping_method.obj is None


def test_multiple_expand(client, commercetools_api):
    shipping_method = client.shipping_methods.create(
        types.ShippingMethodDraft(
            key="test-shipping-method",
            name="test shipping method",
            tax_category=types.TaxCategoryResourceIdentifier(id="dummy"),
            zone_rates=[],
            is_default=False,
        )
    )

    payment = client.payments.create(
        types.PaymentDraft(
            key="test-payment",
            amount_planned=types.Money(cent_amount=2000, currency_code="GBP"),
            payment_method_info=types.PaymentMethodInfo(
                payment_interface="ADYEN", method="mc"
            ),
            transactions=[
                types.TransactionDraft(
                    type=types.TransactionType.CHARGE,
                    amount=types.Money(cent_amount=2000, currency_code="GBP"),
                    state=types.TransactionState.PENDING,
                )
            ],
        )
    )

    order = get_test_order()
    order.shipping_info.shipping_method.id = shipping_method.id
    order.payment_info.payments[0].id = payment.id
    commercetools_api.orders.add_existing(order)

    expanded_order = client.orders.get_by_id(
        order.id, expand=["shippingInfo.shippingMethod", "paymentInfo.payments[*]"]
    )

    assert expanded_order.id
    assert expanded_order.shipping_info.shipping_method.obj.id == shipping_method.id
    assert expanded_order.payment_info.payments[0].obj.id == payment.id

    expanded_order = client.orders.get_by_id(
        order.id, expand=["shippingInfo.shippingMethod"]
    )

    assert expanded_order.id
    assert expanded_order.shipping_info.shipping_method.obj.id == shipping_method.id
    assert expanded_order.payment_info.payments[0].obj is None

    query_results = client.orders.query(
        expand=["shippingInfo.shippingMethod", "paymentInfo.payments[*]"]
    )

    expanded_order = query_results.results[0]

    assert expanded_order.id
    assert expanded_order.shipping_info.shipping_method.obj.id == shipping_method.id
    assert expanded_order.payment_info.payments[0].obj.id == payment.id



import os

import requests

from commercetools import Client, types
from commercetools.types import (
    ChannelDraft,
    ChannelResourceIdentifier,
    ChannelRoleEnum,
    LocalizedString,
    ProductDraft,
    ProductTypeResourceIdentifier,
    StoreDraft,
)


def test_http_server(commercetools_client, commercetools_http_server):
    os.environ["OAUTHLIB_INSECURE_TRANSPORT"] = "1"

    client = Client(
        project_key="unittest",
        client_id="client-id",
        client_secret="client-secret",
        scope=[],
        url=commercetools_http_server.api_url,
        token_url=f"{commercetools_http_server.api_url}/oauth/token",
    )

    query_result = client.products.query()
    assert query_result.count == 0
    product = client.products.create(
        ProductDraft(
            key="test-product",
            product_type=ProductTypeResourceIdentifier(key="dummy"),
            name={"nl": "Testje"},
            slug={"en": "foo-bar"},
        )
    )

    client.products.get_by_id(product.id)
    url = commercetools_http_server.api_url + f"/unittest/products/{product.id}"
    response = requests.get(url, headers={"Authorization": "Bearer token"})

    assert response.status_code == 200, response.text
    data = response.json()
    assert data["masterData"]["staged"]["name"]["nl"] == "Testje"


def test_http_server_expanding(commercetools_client, commercetools_http_server):
    os.environ["OAUTHLIB_INSECURE_TRANSPORT"] = "1"

    client = Client(
        project_key="unittest",
        client_id="client-id",
        client_secret="client-secret",
        scope=[],
        url=commercetools_http_server.api_url,
        token_url=f"{commercetools_http_server.api_url}/oauth/token",
    )

    client.channels.create(
        ChannelDraft(key="FOO", roles=[ChannelRoleEnum.PRODUCT_DISTRIBUTION])
    )

    store = client.stores.create(
        StoreDraft(
            name=types.LocalizedString(nl="foo"),
            key="FOO",
            distribution_channels=[ChannelResourceIdentifier(key="FOO")],
        )
    )

    url = commercetools_http_server.api_url + f"/unittest/stores/{store.id}"
    response = requests.get(
        url,
        params={"expand": "distributionChannels[*]"},
        headers={"Authorization": "Bearer token"},
    )

    assert response.status_code == 200, response.text
    data = response.json()

    assert data["distributionChannels"][0]["obj"]["key"] == "FOO"



# DO NOT EDIT! This file is automatically generated
import datetime
import enum
import typing

from commercetools.types._abstract import _BaseType
from commercetools.types._common import (
    BaseResource,
    Reference,
    ReferenceTypeId,
    ResourceIdentifier,
)

if typing.TYPE_CHECKING:
    from ._common import CreatedBy, LastModifiedBy, LocalizedString, Money, TypedMoney
    from ._tax_category import TaxCategoryReference, TaxCategoryResourceIdentifier
    from ._zone import ZoneReference, ZoneResourceIdentifier
__all__ = [
    "CartClassificationTier",
    "CartScoreTier",
    "CartValueTier",
    "PriceFunction",
    "ShippingMethod",
    "ShippingMethodAddShippingRateAction",
    "ShippingMethodAddZoneAction",
    "ShippingMethodChangeIsDefaultAction",
    "ShippingMethodChangeNameAction",
    "ShippingMethodChangeTaxCategoryAction",
    "ShippingMethodDraft",
    "ShippingMethodPagedQueryResponse",
    "ShippingMethodReference",
    "ShippingMethodRemoveShippingRateAction",
    "ShippingMethodRemoveZoneAction",
    "ShippingMethodResourceIdentifier",
    "ShippingMethodSetDescriptionAction",
    "ShippingMethodSetKeyAction",
    "ShippingMethodSetLocalizedDescriptionAction",
    "ShippingMethodSetPredicateAction",
    "ShippingMethodUpdate",
    "ShippingMethodUpdateAction",
    "ShippingRate",
    "ShippingRateDraft",
    "ShippingRatePriceTier",
    "ShippingRateTierType",
    "ZoneRate",
    "ZoneRateDraft",
]


class PriceFunction(_BaseType):
    #: :class:`str` `(Named` ``currencyCode`` `in Commercetools)`
    currency_code: "str"
    #: :class:`str`
    function: str

    def __init__(self, *, currency_code: "str", function: str) -> None:
        self.currency_code = currency_code
        self.function = function
        super().__init__()

    def __repr__(self) -> str:
        return "PriceFunction(currency_code=%r, function=%r)" % (
            self.currency_code,
            self.function,
        )


class ShippingMethod(BaseResource):
    #: :class:`str`
    id: str
    #: :class:`int`
    version: int
    #: :class:`datetime.datetime` `(Named` ``createdAt`` `in Commercetools)`
    created_at: datetime.datetime
    #: :class:`datetime.datetime` `(Named` ``lastModifiedAt`` `in Commercetools)`
    last_modified_at: datetime.datetime
    #: Optional :class:`commercetools.types.LastModifiedBy` `(Named` ``lastModifiedBy`` `in Commercetools)`
    last_modified_by: typing.Optional["LastModifiedBy"]
    #: Optional :class:`commercetools.types.CreatedBy` `(Named` ``createdBy`` `in Commercetools)`
    created_by: typing.Optional["CreatedBy"]
    #: Optional :class:`str`
    key: typing.Optional[str]
    #: :class:`str`
    name: str
    #: Optional :class:`str`
    description: typing.Optional[str]
    #: Optional :class:`commercetools.types.LocalizedString` `(Named` ``localizedDescription`` `in Commercetools)`
    localized_description: typing.Optional["LocalizedString"]
    #: :class:`commercetools.types.TaxCategoryReference` `(Named` ``taxCategory`` `in Commercetools)`
    tax_category: "TaxCategoryReference"
    #: List of :class:`commercetools.types.ZoneRate` `(Named` ``zoneRates`` `in Commercetools)`
    zone_rates: typing.List["ZoneRate"]
    #: :class:`bool` `(Named` ``isDefault`` `in Commercetools)`
    is_default: bool
    #: Optional :class:`str`
    predicate: typing.Optional[str]

    def __init__(
        self,
        *,
        id: str,
        version: int,
        created_at: datetime.datetime,
        last_modified_at: datetime.datetime,
        name: str,
        tax_category: "TaxCategoryReference",
        zone_rates: typing.List["ZoneRate"],
        is_default: bool,
        last_modified_by: typing.Optional["LastModifiedBy"] = None,
        created_by: typing.Optional["CreatedBy"] = None,
        key: typing.Optional[str] = None,
        description: typing.Optional[str] = None,
        localized_description: typing.Optional["LocalizedString"] = None,
        predicate: typing.Optional[str] = None
    ) -> None:
        self.id = id
        self.version = version
        self.created_at = created_at
        self.last_modified_at = last_modified_at
        self.last_modified_by = last_modified_by
        self.created_by = created_by
        self.key = key
        self.name = name
        self.description = description
        self.localized_description = localized_description
        self.tax_category = tax_category
        self.zone_rates = zone_rates
        self.is_default = is_default
        self.predicate = predicate
        super().__init__(
            id=id,
            version=version,
            created_at=created_at,
            last_modified_at=last_modified_at,
        )

    def __repr__(self) -> str:
        return (
            "ShippingMethod(id=%r, version=%r, created_at=%r, last_modified_at=%r, last_modified_by=%r, created_by=%r, key=%r, name=%r, description=%r, localized_description=%r, tax_category=%r, zone_rates=%r, is_default=%r, predicate=%r)"
            % (
                self.id,
                self.version,
                self.created_at,
                self.last_modified_at,
                self.last_modified_by,
                self.created_by,
                self.key,
                self.name,
                self.description,
                self.localized_description,
                self.tax_category,
                self.zone_rates,
                self.is_default,
                self.predicate,
            )
        )


class ShippingMethodDraft(_BaseType):
    #: Optional :class:`str`
    key: typing.Optional[str]
    #: :class:`str`
    name: str
    #: Optional :class:`str`
    description: typing.Optional[str]
    #: Optional :class:`commercetools.types.LocalizedString` `(Named` ``localizedDescription`` `in Commercetools)`
    localized_description: typing.Optional["LocalizedString"]
    #: :class:`commercetools.types.TaxCategoryResourceIdentifier` `(Named` ``taxCategory`` `in Commercetools)`
    tax_category: "TaxCategoryResourceIdentifier"
    #: List of :class:`commercetools.types.ZoneRateDraft` `(Named` ``zoneRates`` `in Commercetools)`
    zone_rates: typing.List["ZoneRateDraft"]
    #: :class:`bool` `(Named` ``isDefault`` `in Commercetools)`
    is_default: bool
    #: Optional :class:`str`
    predicate: typing.Optional[str]

    def __init__(
        self,
        *,
        name: str,
        tax_category: "TaxCategoryResourceIdentifier",
        zone_rates: typing.List["ZoneRateDraft"],
        is_default: bool,
        key: typing.Optional[str] = None,
        description: typing.Optional[str] = None,
        localized_description: typing.Optional["LocalizedString"] = None,
        predicate: typing.Optional[str] = None
    ) -> None:
        self.key = key
        self.name = name
        self.description = description
        self.localized_description = localized_description
        self.tax_category = tax_category
        self.zone_rates = zone_rates
        self.is_default = is_default
        self.predicate = predicate
        super().__init__()

    def __repr__(self) -> str:
        return (
            "ShippingMethodDraft(key=%r, name=%r, description=%r, localized_description=%r, tax_category=%r, zone_rates=%r, is_default=%r, predicate=%r)"
            % (
                self.key,
                self.name,
                self.description,
                self.localized_description,
                self.tax_category,
                self.zone_rates,
                self.is_default,
                self.predicate,
            )
        )


class ShippingMethodPagedQueryResponse(_BaseType):
    #: Optional :class:`int`
    limit: typing.Optional[int]
    #: :class:`int`
    count: int
    #: Optional :class:`int`
    total: typing.Optional[int]
    #: Optional :class:`int`
    offset: typing.Optional[int]
    #: List of :class:`commercetools.types.ShippingMethod`
    results: typing.Sequence["ShippingMethod"]

    def __init__(
        self,
        *,
        count: int,
        results: typing.Sequence["ShippingMethod"],
        limit: typing.Optional[int] = None,
        total: typing.Optional[int] = None,
        offset: typing.Optional[int] = None
    ) -> None:
        self.limit = limit
        self.count = count
        self.total = total
        self.offset = offset
        self.results = results
        super().__init__()

    def __repr__(self) -> str:
        return (
            "ShippingMethodPagedQueryResponse(limit=%r, count=%r, total=%r, offset=%r, results=%r)"
            % (self.limit, self.count, self.total, self.offset, self.results)
        )


class ShippingMethodReference(Reference):
    #: Optional :class:`commercetools.types.ShippingMethod`
    obj: typing.Optional["ShippingMethod"]

    def __init__(
        self, *, id: str, obj: typing.Optional["ShippingMethod"] = None
    ) -> None:
        self.obj = obj
        super().__init__(type_id=ReferenceTypeId.SHIPPING_METHOD, id=id)

    def __repr__(self) -> str:
        return "ShippingMethodReference(type_id=%r, id=%r, obj=%r)" % (
            self.type_id,
            self.id,
            self.obj,
        )


class ShippingMethodResourceIdentifier(ResourceIdentifier):
    def __init__(
        self, *, id: typing.Optional[str] = None, key: typing.Optional[str] = None
    ) -> None:
        super().__init__(type_id=ReferenceTypeId.SHIPPING_METHOD, id=id, key=key)

    def __repr__(self) -> str:
        return "ShippingMethodResourceIdentifier(type_id=%r, id=%r, key=%r)" % (
            self.type_id,
            self.id,
            self.key,
        )


class ShippingMethodUpdate(_BaseType):
    #: :class:`int`
    version: int
    #: :class:`list`
    actions: list

    def __init__(self, *, version: int, actions: list) -> None:
        self.version = version
        self.actions = actions
        super().__init__()

    def __repr__(self) -> str:
        return "ShippingMethodUpdate(version=%r, actions=%r)" % (
            self.version,
            self.actions,
        )


class ShippingMethodUpdateAction(_BaseType):
    #: :class:`str`
    action: str

    def __init__(self, *, action: str) -> None:
        self.action = action
        super().__init__()

    def __repr__(self) -> str:
        return "ShippingMethodUpdateAction(action=%r)" % (self.action,)


class ShippingRate(_BaseType):
    #: :class:`commercetools.types.TypedMoney`
    price: "TypedMoney"
    #: Optional :class:`commercetools.types.TypedMoney` `(Named` ``freeAbove`` `in Commercetools)`
    free_above: typing.Optional["TypedMoney"]
    #: Optional :class:`bool` `(Named` ``isMatching`` `in Commercetools)`
    is_matching: typing.Optional[bool]
    #: List of :class:`commercetools.types.ShippingRatePriceTier`
    tiers: typing.List["ShippingRatePriceTier"]

    def __init__(
        self,
        *,
        price: "TypedMoney",
        tiers: typing.List["ShippingRatePriceTier"],
        free_above: typing.Optional["TypedMoney"] = None,
        is_matching: typing.Optional[bool] = None
    ) -> None:
        self.price = price
        self.free_above = free_above
        self.is_matching = is_matching
        self.tiers = tiers
        super().__init__()

    def __repr__(self) -> str:
        return "ShippingRate(price=%r, free_above=%r, is_matching=%r, tiers=%r)" % (
            self.price,
            self.free_above,
            self.is_matching,
            self.tiers,
        )


class ShippingRateDraft(_BaseType):
    #: :class:`commercetools.types.Money`
    price: "Money"
    #: Optional :class:`commercetools.types.Money` `(Named` ``freeAbove`` `in Commercetools)`
    free_above: typing.Optional["Money"]
    #: Optional list of :class:`commercetools.types.ShippingRatePriceTier`
    tiers: typing.Optional[typing.List["ShippingRatePriceTier"]]

    def __init__(
        self,
        *,
        price: "Money",
        free_above: typing.Optional["Money"] = None,
        tiers: typing.Optional[typing.List["ShippingRatePriceTier"]] = None
    ) -> None:
        self.price = price
        self.free_above = free_above
        self.tiers = tiers
        super().__init__()

    def __repr__(self) -> str:
        return "ShippingRateDraft(price=%r, free_above=%r, tiers=%r)" % (
            self.price,
            self.free_above,
            self.tiers,
        )


class ShippingRatePriceTier(_BaseType):
    #: :class:`commercetools.types.ShippingRateTierType`
    type: "ShippingRateTierType"

    def __init__(self, *, type: "ShippingRateTierType") -> None:
        self.type = type
        super().__init__()

    def __repr__(self) -> str:
        return "ShippingRatePriceTier(type=%r)" % (self.type,)


class ShippingRateTierType(enum.Enum):
    CART_VALUE = "CartValue"
    CART_CLASSIFICATION = "CartClassification"
    CART_SCORE = "CartScore"


class ZoneRate(_BaseType):
    #: :class:`commercetools.types.ZoneReference`
    zone: "ZoneReference"
    #: List of :class:`commercetools.types.ShippingRate` `(Named` ``shippingRates`` `in Commercetools)`
    shipping_rates: typing.List["ShippingRate"]

    def __init__(
        self, *, zone: "ZoneReference", shipping_rates: typing.List["ShippingRate"]
    ) -> None:
        self.zone = zone
        self.shipping_rates = shipping_rates
        super().__init__()

    def __repr__(self) -> str:
        return "ZoneRate(zone=%r, shipping_rates=%r)" % (self.zone, self.shipping_rates)


class ZoneRateDraft(_BaseType):
    #: :class:`commercetools.types.ZoneResourceIdentifier`
    zone: "ZoneResourceIdentifier"
    #: List of :class:`commercetools.types.ShippingRateDraft` `(Named` ``shippingRates`` `in Commercetools)`
    shipping_rates: typing.List["ShippingRateDraft"]

    def __init__(
        self,
        *,
        zone: "ZoneResourceIdentifier",
        shipping_rates: typing.List["ShippingRateDraft"]
    ) -> None:
        self.zone = zone
        self.shipping_rates = shipping_rates
        super().__init__()

    def __repr__(self) -> str:
        return "ZoneRateDraft(zone=%r, shipping_rates=%r)" % (
            self.zone,
            self.shipping_rates,
        )


class CartClassificationTier(ShippingRatePriceTier):
    #: :class:`str`
    value: str
    #: :class:`commercetools.types.Money`
    price: "Money"
    #: Optional :class:`bool` `(Named` ``isMatching`` `in Commercetools)`
    is_matching: typing.Optional[bool]

    def __init__(
        self, *, value: str, price: "Money", is_matching: typing.Optional[bool] = None
    ) -> None:
        self.value = value
        self.price = price
        self.is_matching = is_matching
        super().__init__(type=ShippingRateTierType.CART_CLASSIFICATION)

    def __repr__(self) -> str:
        return "CartClassificationTier(type=%r, value=%r, price=%r, is_matching=%r)" % (
            self.type,
            self.value,
            self.price,
            self.is_matching,
        )


class CartScoreTier(ShippingRatePriceTier):
    #: :class:`int`
    score: int
    #: Optional :class:`commercetools.types.Money`
    price: typing.Optional["Money"]
    #: Optional :class:`commercetools.types.PriceFunction` `(Named` ``priceFunction`` `in Commercetools)`
    price_function: typing.Optional["PriceFunction"]
    #: Optional :class:`bool` `(Named` ``isMatching`` `in Commercetools)`
    is_matching: typing.Optional[bool]

    def __init__(
        self,
        *,
        score: int,
        price: typing.Optional["Money"] = None,
        price_function: typing.Optional["PriceFunction"] = None,
        is_matching: typing.Optional[bool] = None
    ) -> None:
        self.score = score
        self.price = price
        self.price_function = price_function
        self.is_matching = is_matching
        super().__init__(type=ShippingRateTierType.CART_SCORE)

    def __repr__(self) -> str:
        return (
            "CartScoreTier(type=%r, score=%r, price=%r, price_function=%r, is_matching=%r)"
            % (self.type, self.score, self.price, self.price_function, self.is_matching)
        )


class CartValueTier(ShippingRatePriceTier):
    #: :class:`int` `(Named` ``minimumCentAmount`` `in Commercetools)`
    minimum_cent_amount: int
    #: :class:`commercetools.types.Money`
    price: "Money"
    #: Optional :class:`bool` `(Named` ``isMatching`` `in Commercetools)`
    is_matching: typing.Optional[bool]

    def __init__(
        self,
        *,
        minimum_cent_amount: int,
        price: "Money",
        is_matching: typing.Optional[bool] = None
    ) -> None:
        self.minimum_cent_amount = minimum_cent_amount
        self.price = price
        self.is_matching = is_matching
        super().__init__(type=ShippingRateTierType.CART_VALUE)

    def __repr__(self) -> str:
        return (
            "CartValueTier(type=%r, minimum_cent_amount=%r, price=%r, is_matching=%r)"
            % (self.type, self.minimum_cent_amount, self.price, self.is_matching)
        )


class ShippingMethodAddShippingRateAction(ShippingMethodUpdateAction):
    #: :class:`commercetools.types.ZoneResourceIdentifier`
    zone: "ZoneResourceIdentifier"
    #: :class:`commercetools.types.ShippingRateDraft` `(Named` ``shippingRate`` `in Commercetools)`
    shipping_rate: "ShippingRateDraft"

    def __init__(
        self, *, zone: "ZoneResourceIdentifier", shipping_rate: "ShippingRateDraft"
    ) -> None:
        self.zone = zone
        self.shipping_rate = shipping_rate
        super().__init__(action="addShippingRate")

    def __repr__(self) -> str:
        return (
            "ShippingMethodAddShippingRateAction(action=%r, zone=%r, shipping_rate=%r)"
            % (self.action, self.zone, self.shipping_rate)
        )


class ShippingMethodAddZoneAction(ShippingMethodUpdateAction):
    #: :class:`commercetools.types.ZoneResourceIdentifier`
    zone: "ZoneResourceIdentifier"

    def __init__(self, *, zone: "ZoneResourceIdentifier") -> None:
        self.zone = zone
        super().__init__(action="addZone")

    def __repr__(self) -> str:
        return "ShippingMethodAddZoneAction(action=%r, zone=%r)" % (
            self.action,
            self.zone,
        )


class ShippingMethodChangeIsDefaultAction(ShippingMethodUpdateAction):
    #: :class:`bool` `(Named` ``isDefault`` `in Commercetools)`
    is_default: bool

    def __init__(self, *, is_default: bool) -> None:
        self.is_default = is_default
        super().__init__(action="changeIsDefault")

    def __repr__(self) -> str:
        return "ShippingMethodChangeIsDefaultAction(action=%r, is_default=%r)" % (
            self.action,
            self.is_default,
        )


class ShippingMethodChangeNameAction(ShippingMethodUpdateAction):
    #: :class:`str`
    name: str

    def __init__(self, *, name: str) -> None:
        self.name = name
        super().__init__(action="changeName")

    def __repr__(self) -> str:
        return "ShippingMethodChangeNameAction(action=%r, name=%r)" % (
            self.action,
            self.name,
        )


class ShippingMethodChangeTaxCategoryAction(ShippingMethodUpdateAction):
    #: :class:`commercetools.types.TaxCategoryResourceIdentifier` `(Named` ``taxCategory`` `in Commercetools)`
    tax_category: "TaxCategoryResourceIdentifier"

    def __init__(self, *, tax_category: "TaxCategoryResourceIdentifier") -> None:
        self.tax_category = tax_category
        super().__init__(action="changeTaxCategory")

    def __repr__(self) -> str:
        return "ShippingMethodChangeTaxCategoryAction(action=%r, tax_category=%r)" % (
            self.action,
            self.tax_category,
        )


class ShippingMethodRemoveShippingRateAction(ShippingMethodUpdateAction):
    #: :class:`commercetools.types.ZoneResourceIdentifier`
    zone: "ZoneResourceIdentifier"
    #: :class:`commercetools.types.ShippingRateDraft` `(Named` ``shippingRate`` `in Commercetools)`
    shipping_rate: "ShippingRateDraft"

    def __init__(
        self, *, zone: "ZoneResourceIdentifier", shipping_rate: "ShippingRateDraft"
    ) -> None:
        self.zone = zone
        self.shipping_rate = shipping_rate
        super().__init__(action="removeShippingRate")

    def __repr__(self) -> str:
        return (
            "ShippingMethodRemoveShippingRateAction(action=%r, zone=%r, shipping_rate=%r)"
            % (self.action, self.zone, self.shipping_rate)
        )


class ShippingMethodRemoveZoneAction(ShippingMethodUpdateAction):
    #: :class:`commercetools.types.ZoneResourceIdentifier`
    zone: "ZoneResourceIdentifier"

    def __init__(self, *, zone: "ZoneResourceIdentifier") -> None:
        self.zone = zone
        super().__init__(action="removeZone")

    def __repr__(self) -> str:
        return "ShippingMethodRemoveZoneAction(action=%r, zone=%r)" % (
            self.action,
            self.zone,
        )


class ShippingMethodSetDescriptionAction(ShippingMethodUpdateAction):
    #: Optional :class:`str`
    description: typing.Optional[str]

    def __init__(self, *, description: typing.Optional[str] = None) -> None:
        self.description = description
        super().__init__(action="setDescription")

    def __repr__(self) -> str:
        return "ShippingMethodSetDescriptionAction(action=%r, description=%r)" % (
            self.action,
            self.description,
        )


class ShippingMethodSetKeyAction(ShippingMethodUpdateAction):
    #: Optional :class:`str`
    key: typing.Optional[str]

    def __init__(self, *, key: typing.Optional[str] = None) -> None:
        self.key = key
        super().__init__(action="setKey")

    def __repr__(self) -> str:
        return "ShippingMethodSetKeyAction(action=%r, key=%r)" % (self.action, self.key)


class ShippingMethodSetLocalizedDescriptionAction(ShippingMethodUpdateAction):
    #: Optional :class:`str` `(Named` ``localizedDescription`` `in Commercetools)`
    localized_description: typing.Optional[str]

    def __init__(self, *, localized_description: typing.Optional[str] = None) -> None:
        self.localized_description = localized_description
        super().__init__(action="setLocalizedDescription")

    def __repr__(self) -> str:
        return (
            "ShippingMethodSetLocalizedDescriptionAction(action=%r, localized_description=%r)"
            % (self.action, self.localized_description)
        )


class ShippingMethodSetPredicateAction(ShippingMethodUpdateAction):
    #: Optional :class:`str`
    predicate: typing.Optional[str]

    def __init__(self, *, predicate: typing.Optional[str] = None) -> None:
        self.predicate = predicate
        super().__init__(action="setPredicate")

    def __repr__(self) -> str:
        return "ShippingMethodSetPredicateAction(action=%r, predicate=%r)" % (
            self.action,
            self.predicate,
        )



import datetime
import typing
import uuid

from commercetools import types
from commercetools._schemas._error import ErrorResponseSchema
from commercetools._schemas._subscription import (
    SubscriptionDraftSchema,
    SubscriptionPagedQueryResponseSchema,
    SubscriptionSchema,
    SubscriptionUpdateSchema,
)
from commercetools.testing.abstract import BaseModel, ServiceBackend
from commercetools.testing.utils import create_commercetools_response


class SubscriptionsModel(BaseModel):
    _primary_type_name = "subscription"
    _resource_schema = SubscriptionSchema
    _unique_values = ["key"]

    def _create_from_draft(
        self, draft: types.SubscriptionDraft, id: typing.Optional[str] = None
    ) -> types.Subscription:
        object_id = str(uuid.UUID(id) if id is not None else uuid.uuid4())
        return types.Subscription(
            id=str(object_id),
            version=1,
            created_at=datetime.datetime.now(datetime.timezone.utc),
            last_modified_at=datetime.datetime.now(datetime.timezone.utc),
            key=draft.key,
            changes=draft.changes,
            destination=draft.destination,
            messages=draft.messages,
        )


class SubscriptionsBackend(ServiceBackend):
    service_path = "subscriptions"
    model_class = SubscriptionsModel
    _schema_draft = SubscriptionDraftSchema
    _schema_update = SubscriptionUpdateSchema
    _schema_query_response = SubscriptionPagedQueryResponseSchema

    def urls(self):
        return [
            ("^$", "GET", self.query),
            ("^$", "POST", self.create),
            ("^(?P<id>[^/]+)$", "GET", self.get_by_id),
            ("^(?P<id>[^/]+)$", "POST", self.update_by_id),
            ("^(?P<id>[^/]+)$", "DELETE", self.delete_by_id),
            ("^key=(?P<key>[^/]+)$", "DELETE", self.delete_by_key),
        ]

    def create(self, request):
        obj = self._schema_draft().loads(request.body)

        if isinstance(obj.destination, types.SqsDestination):
            dest = obj.destination
            message = (
                "A test message could not be delivered to this destination: "
                "SQS %r in %r for %r. "
                "Please make sure your destination is correctly configured."
            ) % (dest.queue_url, dest.region, dest.access_key)
            error = types.ErrorResponse(
                status_code=400,
                message=message,
                errors=[types.InvalidInputError(message=message)],
            )
            error_data = ErrorResponseSchema().dumps(error).encode("utf-8")
            return create_commercetools_response(
                request, content=error_data, status_code=400
            )

        data = self.model.add(obj)
        return create_commercetools_response(request, json=data)



# DO NOT EDIT! This file is automatically generated
import datetime
import typing

from commercetools.types._abstract import _BaseType
from commercetools.types._common import (
    BaseResource,
    Reference,
    ReferenceTypeId,
    ResourceIdentifier,
)

if typing.TYPE_CHECKING:
    from ._common import (
        CreatedBy,
        LastModifiedBy,
        LocalizedString,
        Money,
        QueryPrice,
        TypedMoney,
    )
__all__ = [
    "ProductDiscount",
    "ProductDiscountChangeIsActiveAction",
    "ProductDiscountChangeNameAction",
    "ProductDiscountChangePredicateAction",
    "ProductDiscountChangeSortOrderAction",
    "ProductDiscountChangeValueAction",
    "ProductDiscountDraft",
    "ProductDiscountMatchQuery",
    "ProductDiscountPagedQueryResponse",
    "ProductDiscountReference",
    "ProductDiscountResourceIdentifier",
    "ProductDiscountSetDescriptionAction",
    "ProductDiscountSetKeyAction",
    "ProductDiscountSetValidFromAction",
    "ProductDiscountSetValidFromAndUntilAction",
    "ProductDiscountSetValidUntilAction",
    "ProductDiscountUpdate",
    "ProductDiscountUpdateAction",
    "ProductDiscountValue",
    "ProductDiscountValueAbsolute",
    "ProductDiscountValueAbsoluteDraft",
    "ProductDiscountValueDraft",
    "ProductDiscountValueExternal",
    "ProductDiscountValueExternalDraft",
    "ProductDiscountValueRelative",
    "ProductDiscountValueRelativeDraft",
]


class ProductDiscount(BaseResource):
    #: :class:`str`
    id: str
    #: :class:`int`
    version: int
    #: :class:`datetime.datetime` `(Named` ``createdAt`` `in Commercetools)`
    created_at: datetime.datetime
    #: :class:`datetime.datetime` `(Named` ``lastModifiedAt`` `in Commercetools)`
    last_modified_at: datetime.datetime
    #: Optional :class:`commercetools.types.LastModifiedBy` `(Named` ``lastModifiedBy`` `in Commercetools)`
    last_modified_by: typing.Optional["LastModifiedBy"]
    #: Optional :class:`commercetools.types.CreatedBy` `(Named` ``createdBy`` `in Commercetools)`
    created_by: typing.Optional["CreatedBy"]
    #: :class:`commercetools.types.LocalizedString`
    name: "LocalizedString"
    #: Optional :class:`str`
    key: typing.Optional[str]
    #: Optional :class:`commercetools.types.LocalizedString`
    description: typing.Optional["LocalizedString"]
    #: :class:`commercetools.types.ProductDiscountValue`
    value: "ProductDiscountValue"
    #: :class:`str`
    predicate: str
    #: :class:`str` `(Named` ``sortOrder`` `in Commercetools)`
    sort_order: str
    #: :class:`bool` `(Named` ``isActive`` `in Commercetools)`
    is_active: bool
    #: List of :class:`commercetools.types.Reference`
    references: typing.List["Reference"]
    #: Optional :class:`datetime.datetime` `(Named` ``validFrom`` `in Commercetools)`
    valid_from: typing.Optional[datetime.datetime]
    #: Optional :class:`datetime.datetime` `(Named` ``validUntil`` `in Commercetools)`
    valid_until: typing.Optional[datetime.datetime]

    def __init__(
        self,
        *,
        id: str,
        version: int,
        created_at: datetime.datetime,
        last_modified_at: datetime.datetime,
        name: "LocalizedString",
        value: "ProductDiscountValue",
        predicate: str,
        sort_order: str,
        is_active: bool,
        references: typing.List["Reference"],
        last_modified_by: typing.Optional["LastModifiedBy"] = None,
        created_by: typing.Optional["CreatedBy"] = None,
        key: typing.Optional[str] = None,
        description: typing.Optional["LocalizedString"] = None,
        valid_from: typing.Optional[datetime.datetime] = None,
        valid_until: typing.Optional[datetime.datetime] = None
    ) -> None:
        self.id = id
        self.version = version
        self.created_at = created_at
        self.last_modified_at = last_modified_at
        self.last_modified_by = last_modified_by
        self.created_by = created_by
        self.name = name
        self.key = key
        self.description = description
        self.value = value
        self.predicate = predicate
        self.sort_order = sort_order
        self.is_active = is_active
        self.references = references
        self.valid_from = valid_from
        self.valid_until = valid_until
        super().__init__(
            id=id,
            version=version,
            created_at=created_at,
            last_modified_at=last_modified_at,
        )

    def __repr__(self) -> str:
        return (
            "ProductDiscount(id=%r, version=%r, created_at=%r, last_modified_at=%r, last_modified_by=%r, created_by=%r, name=%r, key=%r, description=%r, value=%r, predicate=%r, sort_order=%r, is_active=%r, references=%r, valid_from=%r, valid_until=%r)"
            % (
                self.id,
                self.version,
                self.created_at,
                self.last_modified_at,
                self.last_modified_by,
                self.created_by,
                self.name,
                self.key,
                self.description,
                self.value,
                self.predicate,
                self.sort_order,
                self.is_active,
                self.references,
                self.valid_from,
                self.valid_until,
            )
        )


class ProductDiscountDraft(_BaseType):
    #: :class:`commercetools.types.LocalizedString`
    name: "LocalizedString"
    #: Optional :class:`str`
    key: typing.Optional[str]
    #: Optional :class:`commercetools.types.LocalizedString`
    description: typing.Optional["LocalizedString"]
    #: :class:`commercetools.types.ProductDiscountValueDraft`
    value: "ProductDiscountValueDraft"
    #: :class:`str`
    predicate: str
    #: :class:`str` `(Named` ``sortOrder`` `in Commercetools)`
    sort_order: str
    #: :class:`bool` `(Named` ``isActive`` `in Commercetools)`
    is_active: bool
    #: Optional :class:`datetime.datetime` `(Named` ``validFrom`` `in Commercetools)`
    valid_from: typing.Optional[datetime.datetime]
    #: Optional :class:`datetime.datetime` `(Named` ``validUntil`` `in Commercetools)`
    valid_until: typing.Optional[datetime.datetime]

    def __init__(
        self,
        *,
        name: "LocalizedString",
        value: "ProductDiscountValueDraft",
        predicate: str,
        sort_order: str,
        is_active: bool,
        key: typing.Optional[str] = None,
        description: typing.Optional["LocalizedString"] = None,
        valid_from: typing.Optional[datetime.datetime] = None,
        valid_until: typing.Optional[datetime.datetime] = None
    ) -> None:
        self.name = name
        self.key = key
        self.description = description
        self.value = value
        self.predicate = predicate
        self.sort_order = sort_order
        self.is_active = is_active
        self.valid_from = valid_from
        self.valid_until = valid_until
        super().__init__()

    def __repr__(self) -> str:
        return (
            "ProductDiscountDraft(name=%r, key=%r, description=%r, value=%r, predicate=%r, sort_order=%r, is_active=%r, valid_from=%r, valid_until=%r)"
            % (
                self.name,
                self.key,
                self.description,
                self.value,
                self.predicate,
                self.sort_order,
                self.is_active,
                self.valid_from,
                self.valid_until,
            )
        )


class ProductDiscountMatchQuery(_BaseType):
    #: :class:`str` `(Named` ``productId`` `in Commercetools)`
    product_id: str
    #: :class:`int` `(Named` ``variantId`` `in Commercetools)`
    variant_id: int
    #: :class:`bool`
    staged: bool
    #: :class:`commercetools.types.QueryPrice`
    price: "QueryPrice"

    def __init__(
        self, *, product_id: str, variant_id: int, staged: bool, price: "QueryPrice"
    ) -> None:
        self.product_id = product_id
        self.variant_id = variant_id
        self.staged = staged
        self.price = price
        super().__init__()

    def __repr__(self) -> str:
        return (
            "ProductDiscountMatchQuery(product_id=%r, variant_id=%r, staged=%r, price=%r)"
            % (self.product_id, self.variant_id, self.staged, self.price)
        )


class ProductDiscountPagedQueryResponse(_BaseType):
    #: :class:`int`
    limit: int
    #: :class:`int`
    count: int
    #: Optional :class:`int`
    total: typing.Optional[int]
    #: :class:`int`
    offset: int
    #: List of :class:`commercetools.types.ProductDiscount`
    results: typing.Sequence["ProductDiscount"]

    def __init__(
        self,
        *,
        limit: int,
        count: int,
        offset: int,
        results: typing.Sequence["ProductDiscount"],
        total: typing.Optional[int] = None
    ) -> None:
        self.limit = limit
        self.count = count
        self.total = total
        self.offset = offset
        self.results = results
        super().__init__()

    def __repr__(self) -> str:
        return (
            "ProductDiscountPagedQueryResponse(limit=%r, count=%r, total=%r, offset=%r, results=%r)"
            % (self.limit, self.count, self.total, self.offset, self.results)
        )


class ProductDiscountReference(Reference):
    #: Optional :class:`commercetools.types.ProductDiscount`
    obj: typing.Optional["ProductDiscount"]

    def __init__(
        self, *, id: str, obj: typing.Optional["ProductDiscount"] = None
    ) -> None:
        self.obj = obj
        super().__init__(type_id=ReferenceTypeId.PRODUCT_DISCOUNT, id=id)

    def __repr__(self) -> str:
        return "ProductDiscountReference(type_id=%r, id=%r, obj=%r)" % (
            self.type_id,
            self.id,
            self.obj,
        )


class ProductDiscountResourceIdentifier(ResourceIdentifier):
    def __init__(
        self, *, id: typing.Optional[str] = None, key: typing.Optional[str] = None
    ) -> None:
        super().__init__(type_id=ReferenceTypeId.PRODUCT_DISCOUNT, id=id, key=key)

    def __repr__(self) -> str:
        return "ProductDiscountResourceIdentifier(type_id=%r, id=%r, key=%r)" % (
            self.type_id,
            self.id,
            self.key,
        )


class ProductDiscountUpdate(_BaseType):
    #: :class:`int`
    version: int
    #: :class:`list`
    actions: list

    def __init__(self, *, version: int, actions: list) -> None:
        self.version = version
        self.actions = actions
        super().__init__()

    def __repr__(self) -> str:
        return "ProductDiscountUpdate(version=%r, actions=%r)" % (
            self.version,
            self.actions,
        )


class ProductDiscountUpdateAction(_BaseType):
    #: :class:`str`
    action: str

    def __init__(self, *, action: str) -> None:
        self.action = action
        super().__init__()

    def __repr__(self) -> str:
        return "ProductDiscountUpdateAction(action=%r)" % (self.action,)


class ProductDiscountValue(_BaseType):
    #: :class:`str`
    type: str

    def __init__(self, *, type: str) -> None:
        self.type = type
        super().__init__()

    def __repr__(self) -> str:
        return "ProductDiscountValue(type=%r)" % (self.type,)


class ProductDiscountValueDraft(_BaseType):
    #: :class:`str`
    type: str

    def __init__(self, *, type: str) -> None:
        self.type = type
        super().__init__()

    def __repr__(self) -> str:
        return "ProductDiscountValueDraft(type=%r)" % (self.type,)


class ProductDiscountChangeIsActiveAction(ProductDiscountUpdateAction):
    #: :class:`bool` `(Named` ``isActive`` `in Commercetools)`
    is_active: bool

    def __init__(self, *, is_active: bool) -> None:
        self.is_active = is_active
        super().__init__(action="changeIsActive")

    def __repr__(self) -> str:
        return "ProductDiscountChangeIsActiveAction(action=%r, is_active=%r)" % (
            self.action,
            self.is_active,
        )


class ProductDiscountChangeNameAction(ProductDiscountUpdateAction):
    #: :class:`commercetools.types.LocalizedString`
    name: "LocalizedString"

    def __init__(self, *, name: "LocalizedString") -> None:
        self.name = name
        super().__init__(action="changeName")

    def __repr__(self) -> str:
        return "ProductDiscountChangeNameAction(action=%r, name=%r)" % (
            self.action,
            self.name,
        )


class ProductDiscountChangePredicateAction(ProductDiscountUpdateAction):
    #: :class:`str`
    predicate: str

    def __init__(self, *, predicate: str) -> None:
        self.predicate = predicate
        super().__init__(action="changePredicate")

    def __repr__(self) -> str:
        return "ProductDiscountChangePredicateAction(action=%r, predicate=%r)" % (
            self.action,
            self.predicate,
        )


class ProductDiscountChangeSortOrderAction(ProductDiscountUpdateAction):
    #: :class:`str` `(Named` ``sortOrder`` `in Commercetools)`
    sort_order: str

    def __init__(self, *, sort_order: str) -> None:
        self.sort_order = sort_order
        super().__init__(action="changeSortOrder")

    def __repr__(self) -> str:
        return "ProductDiscountChangeSortOrderAction(action=%r, sort_order=%r)" % (
            self.action,
            self.sort_order,
        )


class ProductDiscountChangeValueAction(ProductDiscountUpdateAction):
    #: :class:`commercetools.types.ProductDiscountValueDraft`
    value: "ProductDiscountValueDraft"

    def __init__(self, *, value: "ProductDiscountValueDraft") -> None:
        self.value = value
        super().__init__(action="changeValue")

    def __repr__(self) -> str:
        return "ProductDiscountChangeValueAction(action=%r, value=%r)" % (
            self.action,
            self.value,
        )


class ProductDiscountSetDescriptionAction(ProductDiscountUpdateAction):
    #: Optional :class:`commercetools.types.LocalizedString`
    description: typing.Optional["LocalizedString"]

    def __init__(
        self, *, description: typing.Optional["LocalizedString"] = None
    ) -> None:
        self.description = description
        super().__init__(action="setDescription")

    def __repr__(self) -> str:
        return "ProductDiscountSetDescriptionAction(action=%r, description=%r)" % (
            self.action,
            self.description,
        )


class ProductDiscountSetKeyAction(ProductDiscountUpdateAction):
    #: Optional :class:`str`
    key: typing.Optional[str]

    def __init__(self, *, key: typing.Optional[str] = None) -> None:
        self.key = key
        super().__init__(action="setKey")

    def __repr__(self) -> str:
        return "ProductDiscountSetKeyAction(action=%r, key=%r)" % (
            self.action,
            self.key,
        )


class ProductDiscountSetValidFromAction(ProductDiscountUpdateAction):
    #: Optional :class:`datetime.datetime` `(Named` ``validFrom`` `in Commercetools)`
    valid_from: typing.Optional[datetime.datetime]

    def __init__(
        self, *, valid_from: typing.Optional[datetime.datetime] = None
    ) -> None:
        self.valid_from = valid_from
        super().__init__(action="setValidFrom")

    def __repr__(self) -> str:
        return "ProductDiscountSetValidFromAction(action=%r, valid_from=%r)" % (
            self.action,
            self.valid_from,
        )


class ProductDiscountSetValidFromAndUntilAction(ProductDiscountUpdateAction):
    #: Optional :class:`datetime.datetime` `(Named` ``validFrom`` `in Commercetools)`
    valid_from: typing.Optional[datetime.datetime]
    #: Optional :class:`datetime.datetime` `(Named` ``validUntil`` `in Commercetools)`
    valid_until: typing.Optional[datetime.datetime]

    def __init__(
        self,
        *,
        valid_from: typing.Optional[datetime.datetime] = None,
        valid_until: typing.Optional[datetime.datetime] = None
    ) -> None:
        self.valid_from = valid_from
        self.valid_until = valid_until
        super().__init__(action="setValidFromAndUntil")

    def __repr__(self) -> str:
        return (
            "ProductDiscountSetValidFromAndUntilAction(action=%r, valid_from=%r, valid_until=%r)"
            % (self.action, self.valid_from, self.valid_until)
        )


class ProductDiscountSetValidUntilAction(ProductDiscountUpdateAction):
    #: Optional :class:`datetime.datetime` `(Named` ``validUntil`` `in Commercetools)`
    valid_until: typing.Optional[datetime.datetime]

    def __init__(
        self, *, valid_until: typing.Optional[datetime.datetime] = None
    ) -> None:
        self.valid_until = valid_until
        super().__init__(action="setValidUntil")

    def __repr__(self) -> str:
        return "ProductDiscountSetValidUntilAction(action=%r, valid_until=%r)" % (
            self.action,
            self.valid_until,
        )


class ProductDiscountValueAbsolute(ProductDiscountValue):
    #: List of :class:`commercetools.types.TypedMoney`
    money: typing.List["TypedMoney"]

    def __init__(self, *, money: typing.List["TypedMoney"]) -> None:
        self.money = money
        super().__init__(type="absolute")

    def __repr__(self) -> str:
        return "ProductDiscountValueAbsolute(type=%r, money=%r)" % (
            self.type,
            self.money,
        )


class ProductDiscountValueAbsoluteDraft(ProductDiscountValueDraft):
    #: List of :class:`commercetools.types.Money`
    money: typing.List["Money"]

    def __init__(self, *, money: typing.List["Money"]) -> None:
        self.money = money
        super().__init__(type="absolute")

    def __repr__(self) -> str:
        return "ProductDiscountValueAbsoluteDraft(type=%r, money=%r)" % (
            self.type,
            self.money,
        )


class ProductDiscountValueExternal(ProductDiscountValue):
    def __init__(self) -> None:
        super().__init__(type="external")

    def __repr__(self) -> str:
        return "ProductDiscountValueExternal(type=%r)" % (self.type,)


class ProductDiscountValueExternalDraft(ProductDiscountValueDraft):
    def __init__(self) -> None:
        super().__init__(type="external")

    def __repr__(self) -> str:
        return "ProductDiscountValueExternalDraft(type=%r)" % (self.type,)


class ProductDiscountValueRelative(ProductDiscountValue):
    #: :class:`int`
    permyriad: int

    def __init__(self, *, permyriad: int) -> None:
        self.permyriad = permyriad
        super().__init__(type="relative")

    def __repr__(self) -> str:
        return "ProductDiscountValueRelative(type=%r, permyriad=%r)" % (
            self.type,
            self.permyriad,
        )


class ProductDiscountValueRelativeDraft(ProductDiscountValueDraft):
    #: :class:`int`
    permyriad: int

    def __init__(self, *, permyriad: int) -> None:
        self.permyriad = permyriad
        super().__init__(type="relative")

    def __repr__(self) -> str:
        return "ProductDiscountValueRelativeDraft(type=%r, permyriad=%r)" % (
            self.type,
            self.permyriad,
        )



# DO NOT EDIT! This file is automatically generated
import marshmallow

from commercetools import helpers, types
from commercetools._schemas._common import (
    BaseResourceSchema,
    ReferenceSchema,
    ResourceIdentifierSchema,
)

__all__ = [
    "LocationSchema",
    "ZoneAddLocationActionSchema",
    "ZoneChangeNameActionSchema",
    "ZoneDraftSchema",
    "ZonePagedQueryResponseSchema",
    "ZoneReferenceSchema",
    "ZoneRemoveLocationActionSchema",
    "ZoneResourceIdentifierSchema",
    "ZoneSchema",
    "ZoneSetDescriptionActionSchema",
    "ZoneSetKeyActionSchema",
    "ZoneUpdateActionSchema",
    "ZoneUpdateSchema",
]


class LocationSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.Location`."""

    country = marshmallow.fields.String()
    state = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.Location(**data)


class ZoneDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ZoneDraft`."""

    key = marshmallow.fields.String(allow_none=True, missing=None)
    name = marshmallow.fields.String(allow_none=True)
    description = marshmallow.fields.String(allow_none=True, missing=None)
    locations = helpers.LazyNestedField(
        nested="commercetools._schemas._zone.LocationSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ZoneDraft(**data)


class ZonePagedQueryResponseSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ZonePagedQueryResponse`."""

    limit = marshmallow.fields.Integer(allow_none=True)
    count = marshmallow.fields.Integer(allow_none=True)
    total = marshmallow.fields.Integer(allow_none=True, missing=None)
    offset = marshmallow.fields.Integer(allow_none=True)
    results = helpers.LazyNestedField(
        nested="commercetools._schemas._zone.ZoneSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ZonePagedQueryResponse(**data)


class ZoneReferenceSchema(ReferenceSchema):
    """Marshmallow schema for :class:`commercetools.types.ZoneReference`."""

    obj = helpers.LazyNestedField(
        nested="commercetools._schemas._zone.ZoneSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.ZoneReference(**data)


class ZoneResourceIdentifierSchema(ResourceIdentifierSchema):
    """Marshmallow schema for :class:`commercetools.types.ZoneResourceIdentifier`."""

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.ZoneResourceIdentifier(**data)


class ZoneSchema(BaseResourceSchema):
    """Marshmallow schema for :class:`commercetools.types.Zone`."""

    id = marshmallow.fields.String(allow_none=True)
    version = marshmallow.fields.Integer(allow_none=True)
    created_at = marshmallow.fields.DateTime(allow_none=True, data_key="createdAt")
    last_modified_at = marshmallow.fields.DateTime(
        allow_none=True, data_key="lastModifiedAt"
    )
    last_modified_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.LastModifiedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="lastModifiedBy",
    )
    created_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.CreatedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="createdBy",
    )
    key = marshmallow.fields.String(allow_none=True, missing=None)
    name = marshmallow.fields.String(allow_none=True)
    description = marshmallow.fields.String(allow_none=True, missing=None)
    locations = helpers.LazyNestedField(
        nested="commercetools._schemas._zone.LocationSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.Zone(**data)


class ZoneUpdateActionSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ZoneUpdateAction`."""

    action = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ZoneUpdateAction(**data)


class ZoneUpdateSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ZoneUpdate`."""

    version = marshmallow.fields.Integer(allow_none=True)
    actions = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("action", "action"),
            discriminator_schemas={
                "addLocation": "commercetools._schemas._zone.ZoneAddLocationActionSchema",
                "changeName": "commercetools._schemas._zone.ZoneChangeNameActionSchema",
                "removeLocation": "commercetools._schemas._zone.ZoneRemoveLocationActionSchema",
                "setDescription": "commercetools._schemas._zone.ZoneSetDescriptionActionSchema",
                "setKey": "commercetools._schemas._zone.ZoneSetKeyActionSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        ),
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ZoneUpdate(**data)


class ZoneAddLocationActionSchema(ZoneUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ZoneAddLocationAction`."""

    location = helpers.LazyNestedField(
        nested="commercetools._schemas._zone.LocationSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ZoneAddLocationAction(**data)


class ZoneChangeNameActionSchema(ZoneUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ZoneChangeNameAction`."""

    name = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ZoneChangeNameAction(**data)


class ZoneRemoveLocationActionSchema(ZoneUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ZoneRemoveLocationAction`."""

    location = helpers.LazyNestedField(
        nested="commercetools._schemas._zone.LocationSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ZoneRemoveLocationAction(**data)


class ZoneSetDescriptionActionSchema(ZoneUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ZoneSetDescriptionAction`."""

    description = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ZoneSetDescriptionAction(**data)


class ZoneSetKeyActionSchema(ZoneUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ZoneSetKeyAction`."""

    key = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ZoneSetKeyAction(**data)
